package api

import (
	"net/http"

	"github.com/gin-gonic/gin"

	"git.nemunai.re/eurocollec/struct"
)

func declareStatsRoutes(router *gin.RouterGroup) {
	router.GET("/stats/coins", coinsStats)
}

func coinsStats(c *gin.Context) {
	s, err := euroc.GetGlobalStats()
	if err != nil {
		c.AbortWithStatusJSON(http.StatusInternalServerError, gin.H{"errmsg": err.Error()})
		return
	}
	c.JSON(http.StatusOK, s)
}

type CountryStatsYear struct {
	TotalHave    int `json:"total_have"`
	TotalExists  int `json:"total_exists"`
	TotalPossess int `json:"total_possess"`
}

type CountryStats struct {
	Years       map[int]*CountryStatsYear `json:"years"`
	TotalHave   int                       `json:"total_have"`
	TotalExists int                       `json:"total_exists"`
}

func countryStats(c *gin.Context) {
	country := c.MustGet("country").(*euroc.Country)

	coins, err := country.GetCoins()
	if err != nil {
		c.AbortWithStatusJSON(http.StatusInternalServerError, gin.H{"errmsg": err.Error()})
		return
	}

	stats := CountryStats{
		Years:       make(map[int]*CountryStatsYear),
		TotalExists: len(coins),
	}

	for _, coin := range coins {
		haves, err := coin.GetHaves()
		if err != nil {
			c.AbortWithStatusJSON(http.StatusInternalServerError, gin.H{"errmsg": err.Error()})
			return
		}

		coin_year := coin.Year.Year()

		if stats.Years[coin_year] == nil {
			stats.Years[coin_year] = new(CountryStatsYear)
		}

		stats.Years[coin_year].TotalExists += 1

		if len(haves) > 0 {
			stats.TotalHave += 1
			stats.Years[coin_year].TotalHave += 1
		}

		stats.Years[coin_year].TotalPossess += len(haves)
	}

	c.JSON(http.StatusOK, stats)
}
