package api

import (
	"net/http"
	"strconv"
	"time"

	"github.com/gin-gonic/gin"

	"git.nemunai.re/eurocollec/struct"
)

func declareYearsRoutes(router *gin.RouterGroup) {
	router.GET("/years", getYears)

	apiYearsRoutes := router.Group("/years/:y")
	apiYearsRoutes.Use(yearHandler)

	apiYearsRoutes.GET("", getCoinsYear)

}

func yearHandler(c *gin.Context) {
	if y, err := strconv.ParseInt(c.Param("y"), 10, 32); err != nil {
		c.AbortWithStatusJSON(http.StatusBadRequest, gin.H{"errmsg": "Not a valid year"})
	} else {
		c.Set("year", int(y))
		c.Next()
	}
}

func getYears(c *gin.Context) {
	thisYear := time.Now().Year()

	var years []int
	for i := thisYear - 2004; i >= 0; i-- {
		years = append(years, 2004+i)
	}

	c.JSON(http.StatusOK, years)
}

func getCoinsYear(c *gin.Context) {
	year := c.MustGet("year").(int)

	coins, err := euroc.GetCoins()
	if err != nil {
		c.AbortWithStatusJSON(http.StatusBadRequest, gin.H{"errmsg": err.Error()})
		return
	}

	var ret []*euroc.Coin

	for _, coin := range coins {
		if coin.Year.Year() == year {
			ret = append(ret, coin)
		}
	}

	c.JSON(http.StatusOK, ret)
}
