package main

import (
	"flag"
	"log"
	"os"
	"os/signal"
	"syscall"

	"git.nemunai.re/eurocollec/struct"
)

func main() {
	var bind = flag.String("bind", "127.0.0.1:8080", "Bind port/socket")
	var dsn = flag.String("dsn", euroc.DSNGenerator(), "DSN to connect to the MySQL server")
	flag.Parse()

	// Sanitize options
	log.Println("Checking paths...")
	if err := sanitizeStaticOptions(); err != nil {
		log.Fatal(err)
	}

	log.Println("Opening database...")
	if err := euroc.DBInit(*dsn); err != nil {
		log.Fatal("Cannot open the database: ", err)
	}
	defer euroc.DBClose()

	log.Println("Creating database...")
	if err := euroc.DBCreate(); err != nil {
		log.Fatal("Cannot create database: ", err)
	}

	a := NewApp()
	go a.Start(*bind)

	// Prepare graceful shutdown
	interrupt := make(chan os.Signal, 1)
	signal.Notify(interrupt, os.Interrupt, syscall.SIGTERM)

	// Wait shutdown signal
	<-interrupt

	log.Print("The service is shutting down...")
	a.Stop()
	log.Println("done")
}
