#!/usr/bin/env perl

use v5.10;
use strict;
use warnings;

use Data::Dumper;

sub treat_chunk($) {
    if ($_[0] =~ /\{\{(.*)\}\}/) {
	my $tr = $1;
	$tr = $tr =~ s/\{\{\s*euro\s*\|([^]]+?)\}\}/$1 euro/rg;
	$tr = $tr =~ s/\{\{([^]]+?)\|anniversaire\}\}/$1 anniversaire/rg;
	$tr = $tr =~ s/\{\{[^]]+?\|([^]]+?)\}\}/$1/rg;
	$tr = $tr =~ s/\{\{[^]]+?\}\}//rg;
	$tr = $tr =~ s/\[\[[^]]+?\|([^]]+?)\]\]/$1/rg;
	$tr = $tr =~ s/\[\[([^]]+?)\]\]/$1/rg;
	$tr = $tr =~ s/<ref.*?>.*?<\/ref>//rg;
	$tr = $tr =~ s/<ref.*?>(\{\{,\}\})?//rg;

	my @spl = split /\s*\|\s*/, $tr;
	return if $spl[0] ne "2 euros commémorative";
	my %ret;
	for my $s (@spl) {
	    if ($s =~ /^(.*?\S)\s*=\s*(\S.*)$/) {
		if ($1 eq "image") {
		    $ret{"image"} = "https://fr.wikipedia.org/wiki/Spécial:FilePath/" . ($2 =~ s/ /_/rg);
		} elsif ($1 eq "tranche") {
		    $ret{"tranche"} = "https://fr.wikipedia.org/wiki/Spécial:FilePath/" . ($2 =~ s/ /_/rg);
		} else {
		    $ret{$1} = $2;
		}
	    }
	}
	return %ret;
    }
}

my %countries = (
    "France" => 1,
	"Allemagne" => 2,
	"Espagne" => 3,
	"Italie" => 4,
	"Portugal" => 5,
	"Irlande" => 6,
	"Pays-Bas" => 7,
	"Andorre" => 8,
	"Autriche" => 9,
	"Belgique" => 10,
	"Chypre" => 11,
	"Estonie" => 12,
	"Finlande" => 13,
	"Grèce" => 14,
	"Lettonie" => 15,
	"Lituanie" => 16,
	"Luxembourg" => 17,
	"Malte" => 18,
	"Monaco" => 19,
	"Saint Marin" => 21,
	"Slovaquie" => 22,
	"Slovénie" => 23,
	"Vatican" => 24,
);

my $chunk = "";
while (<>) {
    chomp;
    $chunk .= $_ . " ";

    my $m = () = ($chunk =~ /\{\{/g);
    my $n = () = ($chunk =~ /\}\}/g);

    if ($m == 0) {
	$chunk = "";
    } elsif ($m - $n == 0) {
	my %r = treat_chunk $chunk;

	if (defined $r{pays} && $countries{$r{pays}}) {
	    print "(";
	    print $countries{$r{pays}};
	    print ",\"";
	    print $r{"thème"} if defined $r{"thème"};
	    print "\",";
	    if (defined $r{"date"} && $r{"date"} =~ /(\w+) (\d+)/) {
		my $year = $2;
		my $month = "01";
		$month = "01" if $1 eq "Janvier";
		$month = "02" if $1 eq "Février";
		$month = "03" if $1 eq "Mars";
		$month = "04" if $1 eq "Avril";
		$month = "05" if $1 eq "Mai";
		$month = "06" if $1 eq "Juin";
		$month = "07" if $1 eq "Juillet";
		$month = "08" if $1 eq "Août";
		$month = "09" if $1 eq "Septembre";
		$month = "10" if $1 eq "Octobre";
		$month = "11" if $1 eq "Novembre";
		$month = "12" if $1 eq "Décembre";

		print "\"$year-$month-01 00:00:00\"";
	    } else {
		print '"1970-01-01 00:00:00"'
	    }
	    print ",\"";
	    print $r{"image"} if defined $r{"image"};
	    print "\",\"";
	    print $r{"description"} if defined $r{"description"};
	    print "\",\"";
	    print $r{"graveur"} if defined $r{"graveur"};
	    print "\",\"";
	    print $r{"tirage"} if defined $r{"tirage"};
	    print "\",\"";
	    print $r{"tranche"} if defined $r{"tranche"};
	    say   "\"),";
	}

	$chunk = "";
    }
}

treat_chunk $chunk;
