String.prototype.capitalize = function() {
    return this
	.toLowerCase()
	.replace(
		/(^|\s|-)([a-z])/g,
	    function(m,p1,p2) { return p1+p2.toUpperCase(); }
	);
}

angular.module("EuroCollecApp", ["ngRoute", "ngResource"])
    .config(function($routeProvider, $locationProvider) {
	$routeProvider
	    .when("/coins", {
		controller: "CoinsController",
		templateUrl: "views/coins.html"
	    })
	    .when("/coins/:coinId", {
		controller: "CoinController",
		templateUrl: "views/coin.html"
	    })
	    .when("/coins/:coinId/edit", {
		controller: "CoinController",
		templateUrl: "views/coin-edit.html"
	    })
	    .when("/countries", {
		controller: "CountriesController",
		templateUrl: "views/countries.html"
	    })
	    .when("/countries/:cname", {
		controller: "CountryController",
		templateUrl: "views/country.html"
	    })
	    .when("/countries/:cname/edit", {
		controller: "CountryController",
		templateUrl: "views/country-edit.html"
	    })
	    .when("/stats", {
		controller: "StatsController",
		templateUrl: "views/stats.html"
	    })
	    .when("/", {
		templateUrl: "views/home.html"
	    });
	$locationProvider.html5Mode(true);
    });

angular.module("EuroCollecApp")
    .factory("Version", function($resource) {
	return $resource("/api/version")
    })
    .factory("Country", function($resource) {
	return $resource("/api/countries/:countryId", { countryId: '@id' }, {
	    'update': {method: 'PUT'},
	})
    })
    .factory("CoinsCountry", function($resource) {
	return $resource("/api/countries/:countryId/coins/:coinId", { countryId: '@id', coinId: '@id' }, {
	    'update': {method: 'PUT'},
	})
    })
    .factory("Coin", function($resource) {
	return $resource("/api/coins/:coinId", { coinId: '@id' }, {
	    'update': {method: 'PUT'},
	})
    })
    .factory("CoinsStats", function($resource) {
	return $resource("/api/stats/coins/:coinId", { coinId: '@id' })
    })
    .factory("CountryStats", function($resource) {
	return $resource("/api/countries/:countryId/stats", { countryId: '@id' })
    })
    .factory("Years", function($resource) {
	return $resource("/api/years/:year", { year: '@id' })
    })
    .factory("HaveCoin", function($resource) {
	return $resource("/api/coins/:coinId/have/:haveId", { coinId: '@id', haveId: '@id' }, {
	    'update': {method: 'PUT'},
	})
    });

angular.module("EuroCollecApp")
    .directive('integer', function() {
	return {
	    require: 'ngModel',
	    link: function(scope, ele, attr, ctrl){
		ctrl.$parsers.unshift(function(viewValue){
		    return parseInt(viewValue, 10);
		});
	    }
	};
    })
    .filter("ucfirst", function() {
	return function(input) {
	    if (input === undefined)
		return input;
	    if (input.length > 1) {
		return input[0].toUpperCase() + input.substring(1);
	    } else if (input.length == 1) {
		return input.toUpperCase();
	    } else {
		return input;
	    }
	}
    })
    .filter("filterByCountry", function() {
        return function(input, country) {
          if (country) {
            return input.filter((coin) => {
              return coin.id_country == country
            })
          } else {
            return input
          }
	}
    })
    .filter("filterByYear", function() {
        return function(input, year) {
          if (year) {
            return input.filter((coin) => {
              return new Date(coin.year).getFullYear() == year
            })
          } else {
            return input
          }
	}
    });

angular.module("EuroCollecApp")
    .run(function($rootScope, $location) {
	$rootScope.goToCountry = function(country) {
	    $location.url("countries/" + country.label);
	    return true;
	};
	$rootScope.goToCoin = function(coin) {
	    $location.url("coins/" + coin.id);
	    return true;
	};
        $rootScope.goToCoins = function(country, year) {
	    $location.url("coins?country=" + country.id + "&year=" + year);
	    return true;
	};
    })
    .controller("VersionController", function($scope, Version) {
	$scope.v = Version.get();
    })
    .controller("StatsController", function($scope, Version) {
      var years = [];

      for (var i = (new Date()).getFullYear() - 2004; i >= 0; i--) {
        years.push(2004 + i);
      }

      $scope.years = years;
    })
    .controller("CoinsStatsController", function($scope, CoinsStats, $routeParams, $rootScope) {
	$scope.coinsStats = CoinsStats.get();
	$rootScope.updateStats = function() {
	    $scope.coinsStats = CoinsStats.get();
	}
    })
    .controller("CountryStatsController", function($scope, CountryStats) {
        $scope.countryStats = CountryStats.get({countryId: $scope.country.id});
    })
    .controller("CoinOfTheYearController", function($scope, Years) {
        $scope.coty = Years.query({year: (new Date).getFullYear()});
    })
    .controller("CoinsController", function($scope, Coin, Country, $routeParams) {
	$scope.coins = Coin.query();
	$scope.countries = {};
	Country.query().$promise.then(function(countries) {
	    countries.forEach(function (country) {
		$scope.countries[country.id] = country;
	    });
	});
        $scope.countryFiltered = $routeParams.country?parseInt($routeParams.country):null;
        $scope.yearFiltered = $routeParams.year?parseInt($routeParams.year):null;

        $scope.filterByCountry = function(country) {
          $scope.countryFiltered = country.id;
        }
        $scope.resetFilterByCountry = function() {
          $scope.countryFiltered = null;
        }

        $scope.filterByYear = function(year) {
          $scope.yearFiltered = parseInt(year);
          console.log($scope.yearFiltered)
        }
        $scope.resetFilterByYear = function() {
          $scope.yearFiltered = null;
        }
    })
    .controller("CoinController", function($scope, Coin, Country, CoinsCountry, $location, $routeParams) {
	$scope.coin = Coin.get({'coinId': $routeParams.coinId });
	$scope.previousCoin = null;
	$scope.nextCoin = null;

	$scope.coin.$promise.then(function(coin) {
	    coin.year = new Date(coin.year);
	    $scope.country = Country.get({'countryId': coin.id_country});

	    $scope.countryCoins = CoinsCountry.query({countryId: coin.id_country});
	    $scope.countryCoins.$promise.then(function(ccs) {
		ccs.forEach(function(cc, k) {
		    if (cc.id == $routeParams.coinId) {
			if (k > 0)
			    $scope.nextCoin = ccs[k-1];
			if (k < ccs.length)
			    $scope.previousCoin = ccs[k+1];
		    }
		})
	    });
	}, function() {
          if ($routeParams.country) {
            $scope.coin.id_country = parseInt($routeParams.country);
	    $scope.country = Country.get({'countryId': $scope.coin.id_country});
          }
        });

	$scope.csubmit = function() {
	    if (this.coin.id) {
		this.coin.$update(function () {
		    $scope.updateStats();
		    $location.url("coins/" + $scope.coin.id)
		}, function(response) {
		    // TODO: display errmsg
		    console.log(response.data);
		});
	    } else {
		this.coin.$save(function (coin) {
		    $scope.updateStats();
		    $location.url("coins/" + coin.id)
		}, function(response) {
		    // TODO: display errmsg
		    console.log(response.data);
		});
	    }
	};
	$scope.delCoin = function() {
	    this.coin.$remove(function () {
		$scope.updateStats();
		$location.url("coins")
	    });
	};
    })
    .controller("CountriesController", function($scope, Country) {
	$scope.countries = Country.query();

	$scope.showCountry = function(country) {
	    country.showCoins = !country.showCoins;
	}
    })
    .controller("CountryController", function($scope, Country, CoinsCountry, $location, $routeParams) {
	$scope.cname = $routeParams.cname;
	if ($routeParams.cname == "_new") {
	    $scope.country = Country.get({"countryId": "_new"});
	} else {
	    var allCountries = Country.query();
	    allCountries.$promise.then(function(countries) {
		countries.forEach(function (country) {
		    if (country.label == $routeParams.cname) {
			$scope.country = Country.get({"countryId": country.id});
			$scope.coins = CoinsCountry.query({countryId: country.id});
			$scope.original_label = country.label;
		    }
		});
	    });
	}

	$scope.csubmit = function() {
	    if (this.country.id) {
		this.country.$update(function () {
		    $location.url("countries/" + $scope.country.label)
		}, function(response) {
		    // TODO: display errmsg
		    console.log(response.data);
		});
	    } else {
		this.country.$save(function () {
		    $location.url("countries/" + $scope.country.label)
		}, function(response) {
		    // TODO: display errmsg
		    console.log(response.data);
		});
	    }
	};
	$scope.delCountry = function() {
	    this.country.$remove(function () {
		$location.url("countries")
	    });
	};
    })
    .controller("CoinsCountryController", function($scope, CoinsCountry) {
	$scope.coins = CoinsCountry.query({countryId: $scope.country.id});
    })
    .controller("HaveCoinsController", function($scope, HaveCoin) {
	$scope.have = HaveCoin.query({coinId: $scope.coin.id});

	$scope.cleaness = {'card': "coin card", 'new': "neuve", 'clean': 'propre', 'used': 'utilisée', 'dirty': 'sale'};

	$scope.addPossession = function() {
	    $scope.have.push({edit: true})
	};
	$scope.editHave = function() {
	    this.h.edit = true;
	};
	$scope.hsubmit = function() {
	    if (this.h.id) {
		this.h.$update({coinId: $scope.coin.id}, function () {
		    $scope.updateStats();
		}, function(response) {
		    // TODO: display errmsg
		    console.log(response.data);
		});
	    } else {
		HaveCoin.save({coinId: $scope.coin.id}, this.h, function () {
		    $scope.updateStats();
		    $scope.have = HaveCoin.query({coinId: $scope.coin.id});
		}, function(response) {
		    // TODO: display errmsg
		    console.log(response.data);
		});
	    }
	};
	$scope.delHave = function() {
	    this.h.$remove({coinId: $scope.coin.id}, function() {
		$scope.have = HaveCoin.query({coinId: $scope.coin.id});
		$scope.updateStats();
	    }, function(response) {
		// TODO: display errmsg
		console.log(response.data);
	    });
	};
    });
