package euroc

import (
	"time"
)

type Coin struct {
	Id               int64     `json:"id"`
	IdCountry        *int64    `json:"id_country"`
	IdCommonIssuance *int64    `json:"id_common_issuance"`
	Label            string    `json:"label"`
	Year             time.Time `json:"year",omitempty`
	Icon             string    `json:"icon",omitempty`
	Description      string    `json:"description",omitempty`
	Engraver         string    `json:"engraver",omitempty`
	Copies           string    `json:"copies",omitempty`
	Slice            string    `json:"slice",omitempty`
}

func GetCoins() (coins []*Coin, err error) {
	if rows, errr := DBQuery("SELECT id_coin, id_country, id_common_issuance, label, year, icon, description, engraver, copies, slice FROM coins ORDER BY year DESC"); errr != nil {
		return nil, errr
	} else {
		defer rows.Close()

		coins = make([]*Coin, 0)
		for rows.Next() {
			var c Coin
			if err = rows.Scan(&c.Id, &c.IdCountry, &c.IdCommonIssuance, &c.Label, &c.Year, &c.Icon, &c.Description, &c.Engraver, &c.Copies, &c.Slice); err != nil {
				return
			}
			coins = append(coins, &c)
		}
		if err = rows.Err(); err != nil {
			return
		}

		return
	}
}

func GetCoin(id int64) (*Coin, error) {
	var c Coin
	err := DBQueryRow("SELECT id_coin, id_country, id_common_issuance, label, year, icon, description, engraver, copies, slice FROM coins WHERE id_coin = ?", id).Scan(&c.Id, &c.IdCountry, &c.IdCommonIssuance, &c.Label, &c.Year, &c.Icon, &c.Description, &c.Engraver, &c.Copies, &c.Slice)
	return &c, err
}

func (country Country) GetCoins() (coins []*Coin, err error) {
	if rows, errr := DBQuery("SELECT id_coin, id_country, id_common_issuance, label, year, icon, description, engraver, copies, slice FROM coins WHERE id_country = ? ORDER BY year DESC", country.Id); errr != nil {
		return nil, errr
	} else {
		defer rows.Close()

		coins = make([]*Coin, 0)
		for rows.Next() {
			var c Coin
			if err = rows.Scan(&c.Id, &c.IdCountry, &c.IdCommonIssuance, &c.Label, &c.Year, &c.Icon, &c.Description, &c.Engraver, &c.Copies, &c.Slice); err != nil {
				return
			}
			coins = append(coins, &c)
		}
		if err = rows.Err(); err != nil {
			return
		}

		return
	}
}

func (country *Country) GetCoin(id int64) (*Coin, error) {
	var c Coin
	err := DBQueryRow("SELECT id_coin, id_country, id_common_issuance, label, year, icon, description, engraver, copies, slice FROM coins WHERE id_coin = ? AND id_country = ?", id, country.Id).Scan(&c.Id, &c.IdCountry, &c.IdCommonIssuance, &c.Label, &c.Year, &c.Icon, &c.Description, &c.Engraver, &c.Copies, &c.Slice)
	return &c, err
}

func NewCoin(country *Country, common_issuance *CommonIssuance, label string, year time.Time, icon string, description string, engraver string, copies string, slice string) (Coin, error) {
	var lid *int64 = nil
	if country != nil {
		lid = &country.Id
	}

	var ccid *int64 = nil
	if common_issuance != nil {
		ccid = &common_issuance.Id
	}

	if res, err := DBExec("INSERT INTO coins (id_country, id_common_issuance, label, year, icon, description, engraver, copies, slice) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)", lid, ccid, label, year, icon, description, engraver, copies, slice); err != nil {
		return Coin{}, err
	} else if cid, err := res.LastInsertId(); err != nil {
		return Coin{}, err
	} else {
		return Coin{cid, lid, ccid, label, year, icon, description, engraver, copies, slice}, nil
	}
}

func (c *Coin) Update() (int64, error) {
	if res, err := DBExec("UPDATE coins SET id_country = ?, id_common_issuance = ?, label = ?, year = ?, icon = ?, description = ?, engraver = ?, copies = ?, slice = ? WHERE id_coin = ?", c.IdCountry, c.IdCommonIssuance, c.Label, c.Year, c.Icon, c.Description, c.Engraver, c.Copies, c.Slice, c.Id); err != nil {
		return 0, err
	} else if nb, err := res.RowsAffected(); err != nil {
		return 0, err
	} else {
		return nb, err
	}
}

func (c *Coin) Delete() (int64, error) {
	if res, err := DBExec("DELETE FROM coins WHERE id_coin = ?", c.Id); err != nil {
		return 0, err
	} else if nb, err := res.RowsAffected(); err != nil {
		return 0, err
	} else {
		return nb, err
	}
}

func ClearCoins() (int64, error) {
	if res, err := DBExec("DELETE FROM coins"); err != nil {
		return 0, err
	} else if nb, err := res.RowsAffected(); err != nil {
		return 0, err
	} else {
		return nb, err
	}
}

func (country *Country) ClearCoins() (int64, error) {
	if res, err := DBExec("DELETE FROM coins WHERE id_country = ?", country.Id); err != nil {
		return 0, err
	} else if nb, err := res.RowsAffected(); err != nil {
		return 0, err
	} else {
		return nb, err
	}
}
