package euroc

import (
	"time"
)

type CommonIssuance struct {
	Id               int64     `json:"id"`
	Label            string    `json:"label"`
	Year             time.Time `json:"year",omitempty`
	Icon             string    `json:"icon",omitempty`
	Description      string    `json:"description",omitempty`
}

func GetCommonIssuances() (ci []CommonIssuance, err error) {
	if rows, errr := DBQuery("SELECT id_common_issuance, label, year, icon, description FROM common_issuances ORDER BY year DESC"); errr != nil {
		return nil, errr
	} else {
		defer rows.Close()

		ci = make([]CommonIssuance, 0)
		for rows.Next() {
			var c CommonIssuance
			if err = rows.Scan(&c.Id, &c.Label, &c.Year, &c.Icon, &c.Description); err != nil {
				return
			}
			ci = append(ci, c)
		}
		if err = rows.Err(); err != nil {
			return
		}

		return
	}
}

func GetCommonIssuance(id int64) (c CommonIssuance, err error) {
	err = DBQueryRow("SELECT id_common_issuance, label, year, icon, description FROM common_issuances WHERE id_common_issuance = ?", id).Scan(&c.Id, &c.Label, &c.Year, &c.Icon, &c.Description)
	return
}

func NewCommonIssuance(label string, year time.Time, icon string, description string) (CommonIssuance, error) {
	if res, err := DBExec("INSERT INTO common_issuances (label, year, icon, description) VALUES (?, ?, ?, ?)", label, year, icon, description); err != nil {
		return CommonIssuance{}, err
	} else if cid, err := res.LastInsertId(); err != nil {
		return CommonIssuance{}, err
	} else {
		return CommonIssuance{cid, label, year, icon, description}, nil
	}
}

func (c CommonIssuance) Update() (int64, error) {
	if res, err := DBExec("UPDATE common_issuances SET label = ?, year = ?, icon = ?, description = ? WHERE id_common_issuance = ?", c.Label, c.Year, c.Icon, c.Description, c.Id); err != nil {
		return 0, err
	} else if nb, err := res.RowsAffected(); err != nil {
		return 0, err
	} else {
		return nb, err
	}
}

func (c CommonIssuance) Delete() (int64, error) {
	if res, err := DBExec("DELETE FROM common_issuances WHERE id_common_issuance = ?", c.Id); err != nil {
		return 0, err
	} else if nb, err := res.RowsAffected(); err != nil {
		return 0, err
	} else {
		return nb, err
	}
}

func ClearCommonIssuance() (int64, error) {
	if res, err := DBExec("DELETE FROM common_issuances"); err != nil {
		return 0, err
	} else if nb, err := res.RowsAffected(); err != nil {
		return 0, err
	} else {
		return nb, err
	}
}
