package euroc

import ()

type Country struct {
	Id    int64  `json:"id"`
	Label string `json:"label"`
	Icon  string `json:"icon,omitempty"`
	Since int    `json:"since"`
}

func GetCountries() (countries []*Country, err error) {
	if rows, errr := DBQuery("SELECT id_country, label, icon, since FROM countries ORDER BY label"); errr != nil {
		return nil, errr
	} else {
		defer rows.Close()

		for rows.Next() {
			var c Country
			if err = rows.Scan(&c.Id, &c.Label, &c.Icon, &c.Since); err != nil {
				return
			}
			countries = append(countries, &c)
		}
		if err = rows.Err(); err != nil {
			return
		}

		return
	}
}

func GetCountry(id int64) (*Country, error) {
	var c Country
	err := DBQueryRow("SELECT id_country, label, icon, since FROM countries WHERE id_country = ?", id).Scan(&c.Id, &c.Label, &c.Icon, &c.Since)
	return &c, err
}

func NewCountry(label string, icon string, since int) (*Country, error) {
	if res, err := DBExec("INSERT INTO countries (label, icon, since) VALUES (?, ?, ?)", label, icon, since); err != nil {
		return nil, err
	} else if cid, err := res.LastInsertId(); err != nil {
		return nil, err
	} else {
		return &Country{cid, label, icon, since}, nil
	}
}

func (c *Country) Update() (int64, error) {
	if res, err := DBExec("UPDATE countries SET label = ?, icon = ?, since = ? WHERE id_country = ?", c.Label, c.Icon, c.Since, c.Id); err != nil {
		return 0, err
	} else if nb, err := res.RowsAffected(); err != nil {
		return 0, err
	} else {
		return nb, err
	}
}

func (c *Country) Delete() (int64, error) {
	if res, err := DBExec("DELETE FROM countries WHERE id_country = ?", c.Id); err != nil {
		return 0, err
	} else if nb, err := res.RowsAffected(); err != nil {
		return 0, err
	} else {
		return nb, err
	}
}

func ClearCountries() (int64, error) {
	if res, err := DBExec("DELETE FROM countries"); err != nil {
		return 0, err
	} else if nb, err := res.RowsAffected(); err != nil {
		return 0, err
	} else {
		return nb, err
	}
}
