package euroc

import (
	"database/sql"
	"log"
	"os"
	"strings"
	"time"

	_ "github.com/go-sql-driver/mysql"
)

// db stores the connection to the database
var db *sql.DB

// DSNGenerator returns DSN filed with values from environment
func DSNGenerator() string {
	db_user := "eurocollec"
	db_password := "eurocollec"
	db_host := ""
	db_db := "eurocollec"

	if v, exists := os.LookupEnv("MYSQL_HOST"); exists {
		db_host = "tcp(" + v + ":"
		if p, exists := os.LookupEnv("MYSQL_PORT"); exists {
			db_host += p + ")"
		} else {
			db_host += "3306)"
		}
	}
	if v, exists := os.LookupEnv("MYSQL_PASSWORD"); exists {
		db_password = v
	} else if v, exists := os.LookupEnv("MYSQL_ROOT_PASSWORD"); exists {
		db_user = "root"
		db_password = v
	}
	if v, exists := os.LookupEnv("MYSQL_USER"); exists {
		db_user = v
	}
	if v, exists := os.LookupEnv("MYSQL_DATABASE"); exists {
		db_db = v
	}

	return db_user + ":" + db_password + "@" + db_host + "/" + db_db
}

// DBInit establishes the connection to the database
func DBInit(dsn string) (err error) {
	if db, err = sql.Open("mysql", dsn+"?parseTime=true&foreign_key_checks=1"); err != nil {
		return
	}

	_, err = db.Exec(`SET SESSION sql_mode = 'STRICT_TRANS_TABLES,NO_ZERO_IN_DATE,NO_ZERO_DATE,ERROR_FOR_DIVISION_BY_ZERO,NO_AUTO_CREATE_USER,NO_ENGINE_SUBSTITUTION';`)
	for i := 0; err != nil && i < 15; i += 1 {
		if _, err = db.Exec(`SET SESSION sql_mode = 'STRICT_TRANS_TABLES,NO_ZERO_IN_DATE,NO_ZERO_DATE,ERROR_FOR_DIVISION_BY_ZERO,NO_AUTO_CREATE_USER,NO_ENGINE_SUBSTITUTION';`); err != nil && i <= 5 {
			log.Println("An error occurs when trying to connect to DB, will retry in 2 seconds: ", err)
			time.Sleep(2 * time.Second)
		}
	}

	return
}

// DBCreate creates all necessary tables used by the package
func DBCreate() (err error) {
	ct := `
CREATE TABLE IF NOT EXISTS countries (id_country INTEGER NOT NULL PRIMARY KEY AUTO_INCREMENT, label VARCHAR(255) NOT NULL, icon VARCHAR(255) NOT NULL, since SMALLINT NOT NULL);
CREATE TABLE IF NOT EXISTS common_issuances (id_common_issuance INTEGER NOT NULL PRIMARY KEY AUTO_INCREMENT, label VARCHAR(255) NOT NULL, year DATETIME, icon VARCHAR(255) NOT NULL, description TEXT);
CREATE TABLE IF NOT EXISTS coins (id_coin INTEGER NOT NULL PRIMARY KEY AUTO_INCREMENT, id_common_issuance INTEGER NULL, id_country INTEGER NOT NULL, label TEXT NOT NULL, year DATETIME, icon VARCHAR(255) NOT NULL, description TEXT, engraver VARCHAR(255), copies VARCHAR(255) NOT NULL, slice VARCHAR(255) NOT NULL, FOREIGN KEY(id_country) REFERENCES countries(id_country), FOREIGN KEY(id_common_issuance) REFERENCES common_issuances(id_common_issuance));
CREATE TABLE IF NOT EXISTS have_coins (id_have INTEGER NOT NULL PRIMARY KEY AUTO_INCREMENT, id_coin INTEGER NOT NULL, cleaness ENUM('card', 'new', 'clean', 'used', 'dirty') NOT NULL DEFAULT 'clean', comment VARCHAR(255) NOT NULL, FOREIGN KEY(id_coin) REFERENCES coins(id_coin));
`
	for _, ln := range strings.Split(ct, "\n") {
		if len(ln) == 0 {
			continue
		}
		if _, err = db.Exec(ln); err != nil {
			return
		}
	}
	return
}

func DBClose() error {
	return db.Close()
}

func DBPrepare(query string) (*sql.Stmt, error) {
	return db.Prepare(query)
}

func DBQuery(query string, args ...interface{}) (*sql.Rows, error) {
	return db.Query(query, args...)
}

func DBExec(query string, args ...interface{}) (sql.Result, error) {
	return db.Exec(query, args...)
}

func DBQueryRow(query string, args ...interface{}) *sql.Row {
	return db.QueryRow(query, args...)
}
