package euroc

import ()

type HaveCoin struct {
	Id       int64  `json:"id"`
	IdCoin   int64  `json:"id_coin"`
	Cleaness string `json:"cleaness"`
	Comment  string `json:"comment",omitempty`
}

func (c *Coin) GetHaves() (haves []*HaveCoin, err error) {
	if rows, errr := DBQuery("SELECT id_have, id_coin, cleaness, comment FROM have_coins WHERE id_coin = ?", c.Id); errr != nil {
		return nil, errr
	} else {
		defer rows.Close()

		haves = make([]*HaveCoin, 0)
		for rows.Next() {
			var h HaveCoin
			if err = rows.Scan(&h.Id, &h.IdCoin, &h.Cleaness, &h.Comment); err != nil {
				return
			}
			haves = append(haves, &h)
		}
		if err = rows.Err(); err != nil {
			return
		}

		return
	}
}

func (c *Coin) GetHave(id int64) (*HaveCoin, error) {
	var h HaveCoin
	err := DBQueryRow("SELECT id_have, id_coin, cleaness, comment FROM have_coins WHERE id_have = ? AND id_coin = ?", id, c.Id).Scan(&h.Id, &h.IdCoin, &h.Cleaness, &h.Comment)
	return &h, err
}

func (coin *Coin) NewHave(cleaness string, comment string) (*HaveCoin, error) {
	if res, err := DBExec("INSERT INTO have_coins (id_coin, cleaness, comment) VALUES (?, ?, ?)", coin.Id, cleaness, comment); err != nil {
		return nil, err
	} else if hid, err := res.LastInsertId(); err != nil {
		return nil, err
	} else {
		return &HaveCoin{hid, coin.Id, cleaness, comment}, nil
	}
}

func (h *HaveCoin) Update() (int64, error) {
	if res, err := DBExec("UPDATE have_coins SET id_coin = ?, cleaness = ?, comment = ? WHERE id_have = ?", h.IdCoin, h.Cleaness, h.Comment, h.Id); err != nil {
		return 0, err
	} else if nb, err := res.RowsAffected(); err != nil {
		return 0, err
	} else {
		return nb, err
	}
}

func (h *HaveCoin) Delete() (string, error) {
	if _, err := DBExec("DELETE FROM have_coins WHERE id_have = ?", h.Id); err != nil {
		return "", err
	} else {
		return "null", err
	}
}

func (c *Coin) ClearHaves() (int64, error) {
	if res, err := DBExec("DELETE FROM have_coins id_coin = ?", c.Id); err != nil {
		return 0, err
	} else if nb, err := res.RowsAffected(); err != nil {
		return 0, err
	} else {
		return nb, err
	}
}

func ClearHaves() (int64, error) {
	if res, err := DBExec("DELETE FROM have_coins"); err != nil {
		return 0, err
	} else if nb, err := res.RowsAffected(); err != nil {
		return 0, err
	} else {
		return nb, err
	}
}
